﻿using System;
using System.Collections.Generic;
using System.Threading;

namespace producerConsumerState
{
    /// <summary>
    /// 
    /// </summary>
    public class BoundedBuffer<T> : AbstractBuffer<T>
    {
        private readonly int _capacity;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="name">The name of the Buffer object</param>
        /// <param name="capacity">The capacity of the Buffer Object. 
        /// Must be a positive number, otherwise ArgumentOutOfRangeException is thrown</param>
        public BoundedBuffer(String name, int capacity)
            : base(name)
        {
            if (capacity <= 0)
            {
                throw new ArgumentOutOfRangeException("Negative size: " + capacity);
            }
            this._capacity = capacity;
        }

        public int Capacity
        {
            get { return _capacity; }
        }

        public Boolean IsFull()
        {
            Monitor.Enter(queue);
            try
            {
                return (queue.Count == _capacity);
            }
            finally
            {
                Monitor.Exit(queue);
            }
        }

        public override void Put(T element)
        {
            if (ending)
            {
                throw new InvalidOperationException("Trying to put after putting the STOP_VALUE");
            }
            Monitor.Enter(queue);
            try
            {
                while (IsFull())
                {
                    Monitor.Wait(queue);
                }
                queue.Enqueue(element);
                //if (queue.Count == 1)
                //{
                    Monitor.PulseAll(queue);
                //}
            }
            finally
            {
                Monitor.Exit(queue);
            }
        }

        public override T take()
        {
            if (ended) { throw new InvalidOperationException("Trying to take after ended ..."); }
            Monitor.Enter(queue);  
            try
            {
                while (IsEmpty() && !ending)
                {
                    Console.WriteLine("Waiting on take");
                    Monitor.Wait(queue);
                }
                T element = queue.Dequeue();

                if (queue.Count == _capacity - 1)
                {
                    Monitor.PulseAll(queue);
                }
                if (queue.Count == 0 && ending)
                {
                    ended = true;
                }
                return element;
            }
            finally
            {
                Monitor.Exit(queue);
            }
        }

    }
}
