﻿using System;

namespace producerConsumer
{
    /// <summary>
    /// Produces items and Puts them into the specified buffer
    /// </summary>
    public class Producer
    {
        private readonly IBuffer _buffer;
        private readonly int _howMany;
        /// <summary>
        /// The last item the producer puts into the buffer.<br />
        /// When consumers sees this item, they know that no more items will follow.
        /// </summary>
        public static int LastItem = -1;

        /// <summary>
        /// Constructs a new object of this class.
        /// </summary>
        /// <param name="buffer">The buffer where the produced elements must be put into</param>
        /// <param name="howMany">The number of element this producer should produce</param>
        /// <exception cref="ArgumentNullException">if buffer is null</exception>
        /// <exception cref="ArgumentOutOfRangeException">if howMany &lt; 0</exception>
        public Producer(IBuffer buffer, int howMany)
        {
            if (buffer == null) { throw new ArgumentNullException("buffer"); }
            if (howMany < 0)
            {
                throw new ArgumentOutOfRangeException("howMany", howMany, "howMany must be a positive number");
            }
            this._buffer = buffer;
            this._howMany = howMany;
        }

        /// <summary>
        /// Runs this Producer<br />
        /// Instantiates the delegate type <code>ThreadStart</code>
        /// </summary>
        public void Run()
        {
            for (int i = 0; i < _howMany; i++)
            {
                _buffer.Put(i);
                Console.WriteLine("Producer put " + i);
            }
            _buffer.Put(LastItem);
        }
    }
}
